function [results results_ind] = STAPLE_online(bind, eind, data_dir, ufn, staple_truth, ...
                                   truth, obs, num_slices, num_labels, ...
                                   num_voxels_per_im, num_voxels, bias, ...
                                   bias_factor, epsilon, type)

% set up arrays to analyze the results
results = zeros([eind - bind + 2, 1]);
result_count = 1;

% run the staple algorithm
[new_truth theta W num_raters] = STAPLE(obs, num_slices, num_labels, ...
                                        num_voxels_per_im, num_voxels, ...
                                        bias, bias_factor, epsilon);
new_truth = double(new_truth);

% calculate the accuracy
tdiff = abs(new_truth - truth);
results(result_count) = (length(tdiff(tdiff == 0))) / length(tdiff);
result_count = result_count + 1;

% initialize the previous results
W_prev = W;
theta_prev = theta;
new_truth_prev = new_truth;

results_ind = zeros(num_slices, eind - bind + 1);

% keep track of which observation we are currently adding
obs_count = 4 * ones([num_slices 1]);

for i = bind:eind
    % display which online iteration we are currently on
    disp(['Beginning Online Iteration Number: ', num2str(i-bind+1)]);

    % Sequential method of adding observations
    if strcmp(type, 'seq')

        % find the sequential observation to add
        u = unique(obs_count);
        if length(u) == 1
            ind = 1;
        else
            ind = find(obs_count == min(u));
            ind = ind(1);
        end

        obs = add_obs(obs, [data_dir, ufn{ind}{obs_count(ind)}], ...
                      num_voxels_per_im);
        obs_count(ind) = obs_count(ind) + 1;

    % Random method of adding observations
    elseif strcmp(type, 'ran')

        % find the random observation to add
        rand_sl = 0;
        while rand_sl < 1
            rand_sl = ceil(num_slices * rand(1));
            if obs_count(rand_sl) == length(ufn{rand_sl}) + 1
                rand_sl = 0;
            end
        end
        obs = add_obs(obs, [data_dir, ufn{rand_sl}{obs_count(rand_sl)}], ...
                      num_voxels_per_im);
        obs_count(rand_sl) = obs_count(rand_sl) + 1;

    % Optimal method of adding observations
    elseif strcmp(type, 'opt')

        % find the probability of the most likely label for each voxel
        max_W = max(W_prev');
        slice_ave_w = zeros([1 num_slices]);

        % find the average probability for each slice
        for s = 1:num_slices
            st_ind = (s - 1) * num_voxels_per_im + 1;
            end_ind = st_ind + num_voxels_per_im - 1;
            slice_ave_w(s) = mean(max_W(st_ind:end_ind));
        end

        % find the optimal slice
        sl = 0;
        while sl <= 0
            % find the voxel with the least amount of certainty
            inds = find(slice_ave_w == min(slice_ave_w));

            % figure out which slice that voxel belongs to
            sl = inds(1);

            if obs_count(sl) == length(ufn{sl}) + 1
                slice_ave_w(sl) = Inf;
                sl = 0;
            end
        end

        % find the optimal observation to add
        obs = add_obs(obs, [data_dir, ufn{sl}{obs_count(sl)}], ...
                      num_voxels_per_im);
        obs_count(sl) = obs_count(sl) + 1;

    % Truth method of adding observations
    elseif strcmp(type, 'tru1')

        % cheating metric, use truth
        tdiff = abs(new_truth_prev - staple_truth);
        tdiff(tdiff ~= 0) = 1;
        slice_err = zeros([1 num_slices]);

        % find the average probability for each slice
        for s = 1:num_slices
            st_ind = (s - 1) * num_voxels_per_im + 1;
            end_ind = st_ind + num_voxels_per_im - 1;
            slice_err(s) = sum(tdiff(st_ind:end_ind));
        end

        % find the optimal slice
        sl = 0;
        while sl <= 0
            % find the voxel with the least amount of certainty
            inds = find(slice_err == max(slice_err));

            % figure out which slice that voxel belongs to
            sl = inds(1);

            if obs_count(sl)== length(ufn{sl}) + 1
                slice_err(sl) = 0;
                sl = 0;
            end
        end

        % find the optimal observation to add
        obs = add_obs(obs, [data_dir, ufn{sl}{obs_count(sl)}], ...
                      num_voxels_per_im);
        obs_count(sl) = obs_count(sl) + 1;

    % Truth method of adding observations
    elseif strcmp(type, 'tru2')

        % initialize all the values to -Inf
        slice_imp = -1 * Inf * ones([num_slices 1]);

        % iterate over each slice
        for sl = 1:num_slices
            % make sure there is a slice to add
            if obs_count(sl) < length(ufn{sl}) + 1
                % find the optimal observation to add
                obs = add_obs(obs, [data_dir, ufn{sl}{obs_count(sl)}], ...
                              num_voxels_per_im);

                % run staple with this added observation
                [new_truth theta W num_raters] = STAPLE(obs, num_slices, ...
                            num_labels, num_voxels_per_im, num_voxels, ...
                            bias, bias_factor, epsilon);
                new_truth = double(new_truth);

                % see how much improvement we get
                tdiff = abs(new_truth - truth);
                slice_imp(sl) = (length(tdiff(tdiff == 0))) / length(tdiff);

                % remove the observation
                obs = rm_obs(obs, obs.num);
            end
        end

        % find which slice provided the most improvement
        inds = find(slice_imp == max(slice_imp));
        sl = inds(1);
        results_ind(:, i-bind+1) = slice_imp;

        % add it to the observations
        obs = add_obs(obs, [data_dir, ufn{sl}{obs_count(sl)}], ...
                      num_voxels_per_im);
        obs_count(sl) = obs_count(sl) + 1;

    end

    % run the staple algorithm
    [new_truth theta W num_raters] = STAPLE(obs, num_slices, num_labels, ...
                                            num_voxels_per_im, num_voxels, ...
                                            bias, bias_factor, epsilon);
    new_truth = double(new_truth);

    tdiff = abs(new_truth - truth);
    results(result_count) = (length(tdiff(tdiff == 0))) / length(tdiff);
    result_count = result_count + 1;

    % reset the previous results
    W_prev = W;
    theta_prev = theta;
    new_truth_prev = new_truth;

    for sl = 1:num_slices
        if obs_count(sl) ~= length(ufn{sl}) + 1
            num = length(ufn{sl});
            rind = obs_count(sl) - 1 + randperm(1 + num - obs_count(sl));
            ufn{sl}(obs_count(sl):end) = ufn{sl}(rind);
        end
    end

    % real-time plot
    %figure(1);
    %clf;
    %plot(results);
    %xlim([1 result_count-1]);
end
