function [results] = coverage_accuracy_theta(monte_carlo_runs, st_cov, ...
                                        end_cov, num_slices, st_slice_truth, ...
                                        num_voxels_per_im, num_voxels, ...
                                        truth_dir, data_dir, st_slice, ...
                                        num_labels, num_volumes, ...
                                        file_reg_ex, ignores, ...
                                        min_ratings, ignore_raters)

% construct the actual truth for determining accuracy
truth = get_estimates(num_slices, st_slice_truth, ...
                      num_voxels_per_im, num_voxels, truth_dir);

% allocate space for the results
results = zeros(monte_carlo_runs, num_slices, end_cov - st_cov + 1);

% gather all of the observations into the cell array
obs_full = get_initial_obs(data_dir, file_reg_ex, ignores, num_voxels_per_im);

% remove unwanted raters
[obs_full ignore_raters] = rm_unwanted_raters(obs_full, ignore_raters, ...
                                              min_ratings);

% construct the ideal theta given the truth and all of the observations
theta = construct_ideal_theta(obs_full, truth, num_slices, num_volumes, ...
                              num_labels, num_voxels_per_im);

% get the observation info
[rater_list, slice_list] = get_obs_info(obs_full);
uniqueraters = unique(rater_list(:, 1));
uniqueslices = unique(slice_list(:, 1));
uniquevolumes = unique(obs_full.volumes);

% get the minimum number of coverages
min_cov = min(slice_list(:, 2));
if min_cov < end_cov
    error(['The ending number of coverages is greater than the ', ...
           'minimum number of coverages']);
end


% run the simulation
for r = 1:monte_carlo_runs
    disp(['Monte Carlo Simulation Number: ', num2str(r)]);

    % make a copy of the observation struct
    obs_edit = obs_full;

    % get a random permutation up to the number of observations
    inds = randperm(obs_full.num);

    % reorder obs_edit
    obs_edit.raters = obs_full.raters(inds);
    obs_edit.slices = obs_full.slices(inds);
    obs_edit.volumes = obs_full.volumes(inds);
    obs_edit.data = obs_full.data(inds);

    % iterate over each coverage amount
    for c = st_cov:end_cov

        % load the observations up to the desired number of coverages
        num_obs = c * num_slices;
        obs = struct('raters', -1 * ones([num_obs 1]), ...
                     'slices', -1 * ones([num_obs 1]), ...
                     'volumes', -1 * ones([num_obs 1]), ...
                     'num', num_obs);
        obs.data = cell(num_obs, 1);

        % find 'c' coverages of all slices
        count = 0;
        for j = 1:num_slices
            for i = 1:obs_edit.num
                if obs_edit.slices(i) == uniqueslices(j)
                    count = count + 1;
                    rater = find(uniqueraters == obs_edit.raters(i));
                    slice = find(uniqueslices == obs_edit.slices(i));
                    volume = find(uniquevolumes == obs_edit.volumes(i));
                    obs.raters(count) = rater(1);
                    obs.slices(count) = slice(1);
                    obs.volumes(count) = volume(1);
                    obs.data(count) = obs_edit.data(i);
                    if ~mod(count, c)
                        break;
                    end
                end
            end
        end

        % calculate W using the ideal theta
        W = calc_w(theta, obs, num_voxels, num_voxels_per_im, num_slices);
        % calculate the staple estimate of the true output for available images
        new_truth = num_labels - ...
                    sum(cumsum(W == repmat(max(W'), [num_labels 1])', 2), 2);

        %view_save_estimates(new_truth, 1, num_slices, '', 70, 110, ...
        %                    num_voxels_per_im, 0, 1);
        %keyboard

        % determine the accuracy of each voxel
        tdiff = abs(new_truth - truth);

        % determine the accuracy of each slice
        for i = 1:num_slices
            bind = (num_voxels_per_im) * (i - 1) + 1;
            eind = num_voxels_per_im * i;
            sl_tdiff = tdiff(bind:eind);
            results(r, i, c - st_cov + 1) = ...
                        (length(sl_tdiff(sl_tdiff == 0))) / length(sl_tdiff);
        end
    end
end

% plot the results
C = {'r', 'g', 'b', 'c', 'm'};
figure(1);
clf;
for i = 1:end_cov-st_cov+1
    x = 1:num_slices;
    y = mean(results(:,:,i));
    e = std(results(:,:,i));
    errorbar(x, y, e, C{i});
    hold on;
end
hold off;
xlabel('Slice Number');
ylabel('% Pixels Correct')
legend({'3 Coverages'; '4 Coverages'; '5 Coverages'; ...
        '6 Coverages'; '7 Coverages'}, 0);
